@@ -3,7 +3,7 @@
 #' Provides access and analytic functions for Sensus data. More information can be found at the
 #' following URL:
 #' 
-#'     https://github.com/MatthewGerber/sensus/wiki
+#'     https://github.com/predictive-technology-laboratory/sensus/wiki
 #' 
 #' @section SensusR functions:
 #' The SensusR functions handle reading, cleaning, plotting, and otherwise analyzing data collected
@@ -21,10 +21,11 @@ NULL
 #' @param local.path Path to location on local machine.
 #' @param aws.path Path to AWS client.
 #' @param delete Whether or not to delete local files that are not present in the S3 path.
+#' @param decompress Whether or not to decompress any gzip files after downloading them.
 #' @return Local path to location of downloaded data.
 #' @examples 
-#' # data.path = sensus.sync.from.aws.s3("s3://bucket/path/to/data", "~/Desktop/data")
-sensus.sync.from.aws.s3 = function(s3.path, profile = "default", local.path = tempfile(), aws.path = "/usr/local/bin/aws", delete = TRUE)
+#' # data.path = sensus.sync.from.aws.s3("s3://bucket/path/to/data", local.path = "~/Desktop/data")
+sensus.sync.from.aws.s3 = function(s3.path, profile = "default", local.path = tempfile(), aws.path = "/usr/local/bin/aws", delete = TRUE, decompress = TRUE)
 {
   aws.args = paste("s3 --profile", profile, "sync ", s3.path, local.path, sep = " ")
   
@@ -34,9 +35,107 @@ sensus.sync.from.aws.s3 = function(s3.path, profile = "default", local.path = te
   }
   
   exit.code = system2(aws.path, aws.args)
+  
+  if(decompress)
+  {
+    sensus.decompress.json(local.path)
+  }
+  
   return(local.path)
 }
 
+#' Decrypts Sensus .bin files that were encrypted using asymmetric public/private key encryption.
+#' 
+#' @param data.path Path to Sensus .bin data (either a file or a directory).
+#' @param is.directory Whether or not the path is a directory.
+#' @param recursive Whether or not to read files recursively from directory indicated by path.
+#' @param rsa.private.key.path Path to RSA private key generated using OpenSSL.
+#' @param rsa.private.key.password Password used to decrypt the RSA private key.
+#' @param replace.files Whether or not to delete .bin files after they have been decrypted.
+#' @return None
+#' @examples
+#' # data.path = system.file("extdata", "example_data", package="SensusR")
+#' # sensus.decrypt.bin.files(data.path = data.path, 
+#' #                          rsa.private.key.path = "/path/to/private.pem", 
+#' #                          replace.files = FALSE)
+sensus.decrypt.bin.files = function(data.path, is.directory = TRUE, recursive = TRUE, rsa.private.key.path, rsa.private.key.password = askpass, replace.files = TRUE)
+{
+  bin.paths = c(data.path)
+  
+  if(is.directory)
+  {
+    bin.paths = list.files(data.path, recursive = recursive, full.names = TRUE, include.dirs = FALSE, pattern = "*.bin")
+  }
+  
+  # read the RSA private key
+  rsa.private.key.file = file(rsa.private.key.path, "rb")
+  rsa.private.key = read_key(rsa.private.key.file, password = rsa.private.key.password)
+  close(rsa.private.key.file)
+  
+  print(paste("Decrypting", length(bin.paths), "file(s)..."))
+  
+  for(bin.path in bin.paths)
+  {
+    bin.file = file(bin.path, "rb")
+    
+    # read/decrypt the symmetric (aes) key
+    enc.aes.key.size = readBin(bin.file, integer(), 1, 4)
+    enc.aes.key = readBin(bin.file, raw(), enc.aes.key.size)
+    aes.key = rsa_decrypt(enc.aes.key, rsa.private.key)
+    
+    # read/decrypt the symmetric (aes) initialization vector
+    enc.aes.iv.size = readBin(bin.file, integer(), 1, 4)
+    enc.aes.iv = readBin(bin.file, raw(), enc.aes.iv.size)
+    aes.iv = rsa_decrypt(enc.aes.iv, rsa.private.key)
+    
+    # read the data content
+    file.size.bytes = file.size(bin.path)
+    data.size.bytes = file.size.bytes - (4 + enc.aes.key.size + 4 + enc.aes.iv.size)
+    enc.data = readBin(bin.file, raw(), data.size.bytes)
+    
+    # make sure we read the rest of the file
+    empty.check = readBin(bin.file, raw(), 1)
+    close(bin.file)
+    
+    if(length(empty.check) != 0)
+    {
+      write("Decryption error:  Leftover bytes in data segment. Proceeding with decryption anyway, but there is something seriously wrong.", stderr())
+    }
+    
+    # decrypt the data using the aes key/iv
+    data = aes_cbc_decrypt(enc.data, aes.key, aes.iv)
+    
+    # write data to decrypted file
+    decrypted.path = sub(".bin$", "", bin.path)
+    decrypted.file = file(decrypted.path, "wb")
+    writeBin(data, decrypted.file)
+    close(decrypted.file)
+    
+    if(replace.files)
+    {
+      file.remove(bin.path)
+    }
+  }
+}
+
+#' Decompresses JSON files downloaded from AWS S3.
+#' 
+#' @param local.path Path to location on local machine.
+#' @return None
+#' @examples 
+#' # sensus.decompress.json("~/Desktop/data")
+sensus.decompress.json = function(local.path)
+{
+  gz.paths = list.files(local.path, recursive = TRUE, full.names = TRUE, include.dirs = FALSE, pattern = "*.gz")
+  
+  print(paste("Decompressing", length(gz.paths), "file(s)..."))
+  
+  for(gz.path in gz.paths)
+  {
+    gunzip(gz.path)
+  }
+}
+
 #' Read JSON-formatted Sensus data.
 #' 
 #' @param data.path Path to Sensus JSON data (either a file or a directory).
@@ -51,6 +150,7 @@ sensus.sync.from.aws.s3 = function(s3.path, profile = "default", local.path = te
 sensus.read.json = function(data.path, is.directory = TRUE, recursive = TRUE, convert.to.local.timezone = TRUE, local.timezone = Sys.timezone())
 {
   paths = c(data.path)
+  
   if(is.directory)
   {
     paths = list.files(data.path, recursive = recursive, full.names = TRUE, include.dirs = FALSE)
@@ -526,6 +626,25 @@ sensus.plot.data.frequency.by.day = function(datum, xlab = "Study Day", ylab = "
   plot(sapply(datum.split.by.day, nrow), xlab = xlab, ylab = ylab, main = main, type = "l", cex.lab = 1.5, cex.axis = 1.5, cex.main = 2)
 }
 
+#' Plot the CDF of inter-reading time lags.
+#' 
+#' @param datum Data frame for a single datum.
+#' @param xlim Limits for the x-axis.
+#' @param xlab Label for x-axis.
+#' @param ylab Label for y-axis.
+#' @param main Label for plot.
+#' @examples 
+#' data.path = system.file("extdata", "example_data", package="SensusR")
+#' data = sensus.read.json(data.path)
+#' sensus.plot.lag.cdf(data$AccelerometerDatum)
+sensus.plot.lag.cdf = function(datum, xlim = c(0,1), xlab = "Inter-reading time (seconds)", ylab = "Percentile", main = paste("Inter-reading times (n=", nrow(datum), ")", sep=""))
+{
+  lags = diff(datum$Timestamp)
+  lag.ecdf = ecdf(as.numeric(lags))
+  num.rows = nrow(datum)
+  plot(lag.ecdf, xlim = xlim, xlab = xlab, ylab = ylab, main = main)
+}
+
 #' Removes all data associated with a device ID from a data collection.
 #' 
 #' @param datum Data collection to process.
